import 'n_printer_status.dart';
import 'n_result.dart';

class NResultPrintable {
  static const int _resultCoverOpened = 0x01;
  static const int _resultOutOfPaper = 0x02;
  static const int _resultPaperJam = 0x04;
  static const int _resultOverheat = 0x08;
  static const int _resultError = 0x0f;
  static const int _resultPowerCableStatus = 0x10;
  static const int _resultChargingStatus = 0x20;
  static const int _resultBatteryStatus = 0x40;
  static const int _resultCommunicationType = 0x80;

  int _result = 0;

  NResultPrintable(int result) {
    _result = result;
  }

  bool isPrintable() {
    return !isInError() && !isBatteryLow();
  }

  bool isInError() {
    return isCoverOpened() || isOutOfPaper() || isPaperJam() || isOverheat();
  }

  int getError() {
    return _result & _resultError;
  }

  int getErrorByPriority() {
    if (isOverheat()) {
      return NPrinterStatus.overheat.code;
    } else if (isCoverOpened()) {
      return NPrinterStatus.coverOpened.code;
    } else if (isOutOfPaper()) {
      return NPrinterStatus.outOfPaper.code;
    } else if (isPaperJam()) {
      return NPrinterStatus.paperJam.code;
    } else {
      return NResult.ok.code;
    }
  }

  bool isCoverOpened() {
    return (_result & _resultCoverOpened) != 0;
  }

  bool isOutOfPaper() {
    return (_result & _resultOutOfPaper) != 0;
  }

  bool isPaperJam() {
    return (_result & _resultPaperJam) != 0;
  }

  bool isOverheat() {
    return (_result & _resultOverheat) != 0;
  }

  bool isPowerCableConnected() {
    return (_result & _resultPowerCableStatus) != 0;
  }

  bool isInCharging() {
    return (_result & _resultChargingStatus) != 0;
  }

  bool isBatteryLow() {
    return (_result & _resultBatteryStatus) != 0;
  }

  bool isCommunicationBle() {
    return (_result & _resultCommunicationType) == 0;
  }

  bool isCommunicationUsb() {
    return (_result & _resultCommunicationType) != 0;
  }
}
